#!/usr/bin/env ruby
lib_path = __FILE__ == '(irb)' ? File.join(Dir.cwd, 'lib') : File.expand_path('../../../lib', __FILE__)
$:.unshift(lib_path) unless $:.include?(lib_path) or !File.exists?(lib_path)
require 'rubygems'
require 'optparse'

require 'envoi/mam/agent/cli'
require 'envoi/mam/iconik/agent'

current_command = ARGV.shift
ARGV << '--help' if ARGV.empty?

default_config_file_paths = Envoi::Mam::Agent::CLI::CONFIG_FILE_PATHS

aspera_ascp_paths = Envoi::Mam::Agent::TransferClient::Aspera::ASCP_PATHS
default_aspera_ascp_path = aspera_ascp_paths.find { |v| File.exist? v } || aspera_ascp_paths.first

# test_args = %w(--download --send-to-envoi --destination-path /tmp)
# ARGV.concat test_args
args = {
  :config_file_path                        => default_config_file_paths,
  :default_ascp_path                       => default_aspera_ascp_path,
  :dry_run                                 => false,
  :ingest_after_upload                     => true,
  :preserve_path                           => true,
  :destination_path                        => File.expand_path('.'),
  :should_download_presentation            => false,
  :should_add_presentation_assets_to_envoi => false
}

args[:config_service_app_id] = ENV['ENVOI_MAM_AGENT_APP_ID']
args[:config_service_app_token] = ENV['ENVOI_MAM_AGENT_APP_TOKEN']
args[:config_service_app_url] = ENV['ENVOI_MAM_AGENT_APP_URL']

op = OptionParser.new
op.on('-c', '--config-file-path FILEPATH', 'The path to the configuration file.') { |v| args[:config_file_path] = v }
op.on('--agent-app-id ID', '') { |v| args[:config_service_app_id] = v }
op.on('--agent-token TOKEN', '') { |v| args[:config_service_app_token] = v }
op.on('--agent-service-api URL', '') { |v| args[:config_service_api_url] = v }

op.on('--asset-id ASSETID', 'The Iconik item ID.') { |v| args[:asset_id] = v }
op.on('-f', '--format-name FORMATNAME', 'The Iconik format name used to find the file to download.') { |v| args[:format_name] = v }
op.on('--storage-id STORAGEID', 'The Iconik storage id to use for the transfer.') { |v| args[:storage_id] = v}


op.on('--file-path PATH', '') { |v| args[:file_path] = v }
op.on('-d', '--destination-path DIRPATH', 'The destination path to download or upload to.',
      "default: '#{args[:destination_path]}'") { |v| args[:destination_path] = v }

op.on('-p','--[no-]preserve-path', "default: #{args[:preserve_path]}") { |v| args[:preserve_path] = v }

op.on('-o', '--operation OP', [ :download, :upload ], 'Transfer Direction.',
      "default: #{args[:operation]}" ) { |v| args[:operation] = v }
op.on('--transfer-type TYPE', [ :aspera, :s3 ],'Force the type of transfer to use. Ex: aspera || s3') { |v| args[:transfer_type] = v }


op.on('-i', '--[no-]import', 'Determines if an asset is created after a upload completes.',
      "default: #{args[:ingest_after_upload]}") { |v| args[:ingest_after_upload] = v}
op.on('--[no-]analyze-asset', 'Determines if an analyze job is started on an asset after it is imported',
      "default: #{args[:analyze_asset]}") { |v| args[:analyze_asset] = v }

# op.on('--dry-run', 'Run without executing the transfer.') { |v| args[:dry_run] = v }
op.on('--help', 'Show this message.') { puts op; exit }
op.load
op.parse!

args[:file_path] ||= ARGV.shift
args[:destination_path] ||= ARGV.shift unless ARGV.empty?

agent = Envoi::Mam::Iconik::Agent.load_config_and_init(args)
agent.run_operation
