#!/usr/bin/env ruby
lib_path = __FILE__ == '(irb)' ? File.join(Dir.cwd, 'lib') : File.expand_path('../../../lib', __FILE__)
$:.unshift(lib_path) unless $:.include?(lib_path) or !File.exists?(lib_path)
require 'rubygems'
require 'optparse'
require 'time'

require 'envoi/mam/agent/cli'
require 'envoi/utils/mediaconvert_enumerator'

DEFAULT_LIST_JOBS_STATUS = 'ERROR'

args = {
  job_status: DEFAULT_LIST_JOBS_STATUS
}
op = OptionParser.new
op.accept(Time) { |time_in| Time.parse(time_in) }
op.on('--aws-profile PROFILENAME', 'AWS credentials file profile name to use.') { |v| args[:aws_profile] = v }
op.on('--aws-access-key KEY', 'AWS access ley') { |v| args[:aws_access_key_id] = v }
op.on('--aws-secret-key KEY', 'AWS secret access key') { |v| args[:aws_secret_access_key] = v }
op.on('--aws-region REGION', 'The AWS region the jobs are located.') { |v| args[:aws_region] = v }
op.on('--new-role ROLE', 'The new role arn to use when resubmitting jobs.') { |v| args[:new_role] = v }
op.on('--new-queue QUEUE', 'The new queue arn to use when resubmitting jobs.') { |v| args[:new_queue] = v }
op.on('--[no-]job-status [STATUS]', 'A job status to query for.',
      "default: '#{args[:job_status]}'") { |v| args[:job_status] = v }
op.on('--from-date DATE', Time, 'The earliest date to filter jobs to.') { |v| args[:from_date] = v }
op.on('--to-date DATE', Time,'The latest date to filter jobs to.') { |v| args[:to_date] = v }
op.on('--limit LIMIT', 'Limit the number of jobs to process.') { |v| args[:limit] = v }
op.on('--verbose', '') { |v| args[:verbose] = v }
op.on('--[no-]preview', 'Output paths with summary and stop.',
      "default: #{args[:preview_mode] ? 'true' : 'false'}") { |v| args[:preview_only] = v }
op.on('--[no-]summary', 'Output the paths and calculate the number of assets.',
      "default: #{args[:show_summary] ? 'true' : 'false'}",) { |v| args[:show_summary] = v }

op.on('-h', '--help', 'Print help (this message) and exit') { puts op; exit }
op.parse!

new_queue = args[:new_queue]
new_role = args[:new_role]

mc_enum = MediaConvertEnumerator.new(args)
mc_enum.run
exit if args[:preview_only]

puts "\n\n"

jobs_to_retry = [ ]
mc_enum.jobs_by_error_code.each do |error_code, jobs|
  jobs.each do |job|
    jobs_to_retry << job
  end
end

verbose = args[:verbose]

job_keys_to_keep = [ :queue, :role, :settings, :user_metadata ]
job_counter = 0
total_jobs_to_retry = jobs_to_retry.length

jobs_to_retry.each do |job|
  job_counter += 1
  input_file = job.settings.inputs.first.file_input rescue ''
  puts "Processing #{job_counter} of #{total_jobs_to_retry} #{job.arn} '#{input_file}'"
  new_job = job.to_h
  new_job.keep_if { |k,v| job_keys_to_keep.include?(k) }
  new_job[:queue] = new_queue if new_queue
  new_job[:role] = new_role if new_role

  resp = mc_enum.media_convert.create_job(new_job)
  puts "Response: #{resp}" if verbose
end

