#!/usr/bin/env ruby
# lib_path = __FILE__ == '(irb)' ? File.join(Dir.cwd, 'lib') : File.expand_path('../../../lib', __FILE__)
# $:.unshift(lib_path) unless $:.include?(lib_path) or !File.exists?(lib_path)
# require 'rubygems'
require 'optparse'
require 'pp'

require 'envoi/mam/agent/cli'
require 'envoi/mam/mediasilo/agent'

current_command = ARGV.shift
ARGV << '--help' if ARGV.empty?

default_config_file_paths = Envoi::Mam::Agent::CLI::CONFIG_FILE_PATHS

aspera_ascp_paths = Envoi::Mam::Agent::TransferClient::Aspera::ASCP_PATHS
default_aspera_ascp_path = aspera_ascp_paths.find { |v| File.exist? v } || aspera_ascp_paths.first

args = {
    :config_file_path => default_config_file_paths,
    :default_aspera_ascp_path => default_aspera_ascp_path,
    :dry_run => false,
    :operation => :download,
    :preserve_path => true,
    :transfer_type => '',

    :asset_derivative_type => 'source',
}

args[:config_service_app_id] = ENV['ENVOI_MAM_AGENT_APP_ID']
args[:config_service_app_token] = ENV['ENVOI_MAM_AGENT_APP_TOKEN']
args[:config_service_app_url] = ENV['ENVOI_MAM_AGENT_APP_URL']

op = OptionParser.new
op.on('--mediasilo-path PATH', 'The path on MediaSilo to download from or upload to.', "PROJECTNAME/[FOLDERNAME]/.../[ASSETTITLE]") { |v| args[:mediasilo_path] = v }
op.on('--mediasilo-project-name PROJECTNAME') { |v| args[:mediasilo_project_name] = v }
op.on('--mediasilo-asset-derivative-type DERIVATIVE', 'The derivative to use when downloading and asset.', "default: #{args[:asset_derivative_type]}") { |v| args[:asset_derivative_type] = v }
op.on('--mediasilo-asset-id ASSETID', '') { |v| args[:asset_id] = v }

op.on('-c', '--config-file-path FILEPATH', 'The path to the configuration file.') { |v| args[:config_file_path] = v }
op.on('--agent-app-id ID', '') { |v| args[:config_service_app_id] = v }
op.on('--agent-token TOKEN', '') { |v| args[:config_service_app_token] = v }
op.on('--agent-service-api URL', '') { |v| args[:config_service_api_url] = v }

op.on('--file-path PATH', '') { |v| args[:file_path] = v }
op.on('-d', '--destination-path DIRPATH', 'The destination path to download or upload to.', "default: '.'") { |v| args[:destination_path] = v }
op.on('-p','--[no-]preserve-path', "default: true") { |v| args[:preserve_path] = v }
op.on('-o', '--operation OP', [ :download, :upload ], 'Transfer Direction.', "default: #{args[:operation]}" ) { |v| args[:operation] = v }
op.on('--transfer-type TYPE', [ :aspera, :s3 ],'Force the type of transfer to use. Ex: aspera || s3') { |v| args[:transfer_type] = v }
op.on('--transfer-token TOKEN', '') { |v| args[:transfer_token] = v }
op.on('--dry-run', 'Run without executing the transfer.') { |v| args[:dry_run] = v }
op.on('--help', 'Show this message.') { puts op; exit }
op.load
op.parse!

args[:file_path] ||= ARGV.shift
args[:destination_path] ||= ARGV.shift unless ARGV.empty?

# module Envoi
#
#   module Mam
#
#     class MediaSilo
#
#       attr_accessor :logger, :config, :api_client
#
#       class Agent < Envoi::Mam::Agent
#
#         DEFAULT_DOWNLOAD_DESTINATION_PATH = '.'
#
#         class CaseSensitiveHeaderKey < String
#           # def downcase; self end
#           def capitalize; self end
#         end
#
#         class FileToSend < File
#           def bytesize; size end
#         end
#
#         def initialize_api_client(args = { })
#           @api_client = args[:api_client] || begin
#             ms_config = config['mediasilo']
#             mediasilo_hostname = ms_config['hostname']
#             mediasilo_username = ms_config['username']
#             mediasilo_password = ms_config['password']
#             mediasilo_api_key = ms_config['api_key']
#
#             client_args = { }
#             client_args[:hostname] = mediasilo_hostname if mediasilo_hostname
#             client_args[:username] = mediasilo_username if mediasilo_username
#             client_args[:password] =  mediasilo_password if mediasilo_password
#             client_args[:api_key] = mediasilo_api_key if mediasilo_api_key
#             Ubiquity::MediaSilo::API::V3::Utilities.new(client_args)
#           end
#
#         end
#
#         def download(args = { })
#           transfer_type = args[:transfer_type]
#
#           file_path = args[:file_path]
#           project_id = args[:project_id]
#           folder_id = args[:folder_id]
#           asset_id = args[:asset_id]
#
#           mediasilo_path = args[:mediasilo_path]
#           project_name = args[:project_name]
#           folder_name = args[:folder_name]
#
#           if mediasilo_path || project_name || folder_name
#             check_path_result = api_client.check_path(mediasilo_path)
#             found = check_path_result[:existing]
#
#             project = found[:project] || { }
#             project_id = project['id']
#
#             folders = found[:folders]
#             folder_id = (folders.last || { })['id']
#
#             asset = found[:asset] || { }
#             asset_id = asset['id']
#           end
#
#           asset ||= asset_id
#
#           destination_file_path = args[:destination_path] || DEFAULT_DOWNLOAD_DESTINATION_PATH
#
#           case transfer_type
#           when :aspera; download_using_aspera(asset_id, destination_file_path, args)
#           else
#             do_upload_response = download_using_http(asset, destination_file_path, args)
#             asset_url = do_upload_response[:asset_url]
#           end
#
#         end
#
#         def download_using_aspera(asset_id, destination_file_path, args = { })
#           derivative_type = args[:asset_derivative_type]
#           derivative_type = derivative_type.downcase == 'source' ? 'source' : 'proxy'
#           ticket = api.aspera_file_download_ticket_create(:asset_id => asset_id, :target => derivative_type)
#
#           host = ticket['server']
#           username = ticket['username']
#           password = ticket['password']
#           token = ticket['token']
#
#           original_file_name = ticket['fileName']
#
#           file_path = ticket['path']
#           target_path = destination_file_path
#
#           FileUtils.mkdir_p(target_path) if target_path.end_with?('/') && !File.directory?(target_path)
#           if target_path.end_with?('/') || File.directory?(target_path)
#             target_path = File.join(target_path, original_file_name)
#           end
#
#           aspera_config = { }
#           aspera_config['host'] = host
#           aspera_config['username'] = username
#           aspera_config['password'] = password
#           aspera_config['token'] = token
#
#           client = Envoi::Mam::Agent::TransferClient::Aspera.new(agent: self)
#           client.download(aspera_config, file_path, target_path)
#         end
#
#         def download_using_http(asset, destination_file_path, args)
#           overwrite = args.fetch(:overwrite, false)
#           derivative_type = args[:asset_derivative_type]
#           api_client.asset_download_derivative(derivative_type, asset, destination_file_path, overwrite)
#         end
#
#         def upload(args = { })
#           transfer_type = args[:transfer_type]
#
#           file_path = args[:file_path]
#           project_id = args[:project_id]
#           folder_id = args[:folder_id]
#           asset_id = args[:asset_id]
#
#           mediasilo_path = args[:mediasilo_path]
#           project_name = args[:project_name]
#           folder_name = args[:folder_name]
#
#           if mediasilo_path || project_name || folder_name
#             check_path_result = api_client.check_path(mediasilo_path, true)
#             found = check_path_result[:existing]
#
#             project = found[:project] || { }
#             project_id = project['id']
#
#             folders = found[:folders]
#             folder_id = (folders.last || { })['id']
#           end
#
#           # # preserve_path = args.fetch(:preserve_path, vidispine_storage_true)
#           # preserve_path = args.fetch(:preserve_path, true)
#           #
#           # destination_path = args[:destination_path] || DEFAULT_DESTINATION_PATH
#           # relative_path = preserve_path ? File.dirname(file_path) : nil
#           # relative_path = nil if relative_path == '.'
#           #
#           # target_path = relative_path ? File.join(destination_path, relative_path) : destination_path
#           # target_path = target_path[0..-1] if target_path.start_with?('/') && !destination_path.start_with?('/')
#
#           # upload file
#           case transfer_type
#           when :aspera; upload_using_aspera(file_path, args)
#           else
#             do_upload_response = upload_using_http(file_path)
#             asset_url = do_upload_response[:asset_url]
#           end
#
#           asset_create_args = {
#               'projectId' => project_id,
#               'folderId' => folder_id,
#               :source_url => asset_url
#           }
#           response = api_client.asset_create(asset_create_args)
#         end
#
#         def upload_using_aspera(file_path, args = { })
#           raise 'Upload using Aspera not yet Implemented.'
#
#           file_name = File.basename(file_path)
#
#           ticket =  api.aspera_file_upload_ticket_create(:file_name => file_name)
#
#           host = ticket['server']
#           username = ticket['username']
#           password = ticket['password']
#           token = ticket['token']
#           destination = ticket['destination']
#
#           target_path = destination
#
#           aspera_config = { }
#           aspera_config['host'] = host
#           aspera_config['username'] = username
#           aspera_config['password'] = password
#           aspera_config['token'] = token
#
#           client = Envoi::Mam::Agent::TransferClient::Aspera.new(agent: self)
#           client.upload(aspera_config, file_path, target_path)
#         end
#
#         def upload_using_http(file_path)
#           file_name = File.basename(file_path)
#
#           res = api_client.asset_upload_ticket_create(:file_name => file_name)
#           uri_str = res['assetUrl']
#
#           uri = URI.parse(uri_str)
#
#           amz_date = res['amzDate']
#           amz_acl = res['amzAcl']
#           content_type = res['contentType']
#           authorization = res['authorization']
#
#           file = File.open(file_path)
#           # file = FileToSend.open(file_path)
#
#           req = Net::HTTP::Put.new(uri.request_uri)
#           req['x-amz-date'] = amz_date
#           req['x-amz-acl'] = amz_acl
#           req[CaseSensitiveHeaderKey.new('Content-Type')] = content_type
#           res[CaseSensitiveHeaderKey.new('Authorization')] = authorization
#           req[CaseSensitiveHeaderKey.new('Transfer-Encoding')] = 'chunked'
#           req[CaseSensitiveHeaderKey.new('Content-Length')] = file.size
#
#           req.body_stream = file
#
#           # req.body = FileToSend.open(file_path)
#
#           request = req
#           http = Net::HTTP.new(uri.host, uri.port)
#           http.use_ssl = true
#           logger.debug { %(REQUEST: #{request.method} https://#{http.address}:#{http.port}#{request.path} HEADERS: #{request.to_hash.inspect} #{api_client.http_client.log_request_body and request.request_body_permitted? ? "BODY: #{api_client.http_client.format_body_for_log_output(request)}" : ''}) }
#
#           res = http.request(req)
#           { :asset_url => uri_str }
#         end
#
#
#       end
#
#     end
#
#   end
#
# end

agent = Envoi::Mam::MediaSilo::Agent.load_config_and_init(args)
agent.run_operation


