#!/usr/bin/env ruby
lib_path = __FILE__ == '(irb)' ? File.join(Dir.cwd, 'lib') : File.expand_path('../../../lib', __FILE__)
$:.unshift(lib_path) unless $:.include?(lib_path) or !File.exists?(lib_path)
require 'rubygems'
require 'optparse'
require 'open3'

require 'envoi/mam/agent/cli'
require 'envoi/restore/agent'

default_config_file_paths = Envoi::Mam::Agent::CLI::CONFIG_FILE_PATHS

aspera_ascp_paths = Envoi::Mam::Agent::TransferClient::Aspera::ASCP_PATHS
default_aspera_ascp_path = aspera_ascp_paths.find { |v| File.exist? v } || aspera_ascp_paths.first

current_command = ARGV.shift
args = {
    :config_file_path  => default_config_file_paths,
    :default_ascp_path => default_aspera_ascp_path,
    :dry_run           => false,
    :operation         => :download,
    :preserve_path     => true,
    :transfer_type     => '',
}

args[:config_service_app_id] = ENV['ENVOI_MAM_AGENT_APP_ID']
args[:config_service_app_token] = ENV['ENVOI_MAM_AGENT_APP_TOKEN']
args[:config_service_app_url] = ENV['ENVOI_MAM_AGENT_APP_URL']

op = OptionParser.new
op.on('-c', '--config-file-path FILEPATH', 'The path to the configuration file.',
      "Default Path(s): #{args[:config_file_path]}") { |v| args[:config_file_path] = v }
# op.on('--agent-app-id ID', '') { |v| args[:config_service_app_id] = v }
# op.on('--agent-token TOKEN', '') { |v| args[:config_service_app_token] = v }
# op.on('--agent-service-api URL', '') { |v| args[:config_service_api_url] = v }

op.on('-h', '--help', 'Show this message.') { puts op; exit }
op.load
op.parse!

begin
  agent = Envoi::Restore::Agent.load_config_and_init(args)
rescue => e
  abort(e.message)
end

begin
  agent.run
rescue Interrupt
  agent.stop
end

