#! /usr/bin/env ruby
# Installation
# `gem install aws-sdk-s3`
#
require 'optparse'
require 'pp'
require 'csv'

require 'envoi/utils/s3_enumerator'
require 'envoi/ingest_service/client'


ARGV << '--help' if ARGV.empty?
args = {
  show_summary: true,
  eis_url: 'https://6bxyr9mdti.execute-api.us-east-1.amazonaws.com/latest/createElemental',
  s3_object_key_prefix: '',
  preview_only: false
}
op = OptionParser.new
op.on('--app-name [APPNAME]', '' ) { |v| args[:app_name] = v }
op.on('--source-bucket [BUCKETNAME]', 'The bucket to get objects from.') { |v| args[:s3_source_bucket_name] = v }
op.on('--target-bucket [BUCKETNAME]', '') { |v| args[:s3_target_bucket_name] = v }
op.on('--aws-access-key [KEY]', '') { |v| args[:aws_access_key_id] = v }
op.on('--aws-secret-key [KEY]', '') { |v| args[:aws_secret_access_key] = v }
op.on('--aws-profile [PROFILENAME]', '') { |v| args[:aws_profile] = v }
op.on('--aws-region [REGION]', 'The AWS region what the bucket is located.') { |v| args[:aws_region] = v }
op.on('--limit [LIMIT]', 'Limit the number of results') { |v| args[:limit] = v }
op.on('--key-prefix [KEYPREFIX]', 'The object key prefix to limit the results by.') { |v| args[:s3_object_key_prefix] = v }
op.on('--eis-url [URL]', 'The URL for the Envoi Ingest Service.',
      "default: #{args[:eis_url]}") { |v| args[:eis_url] = v }
op.on('--[no-]allow-reimport', 'Determines if files existing in the database will be re-imported.') { |v| args[:allow_reimport] = v }
op.on('--[no-]preview', 'Output paths with summary and stop.',
      "default: #{args[:preview_mode] ? 'true' : 'false'}") { |v| args[:preview_only] = v }
op.on('--[no-]summary', 'Output the paths and calculate the number of assets.',
      "default: #{args[:show_summary] ? 'true' : 'false'}",) { |v| args[:show_summary] = v }

op.on('-h', '--help', 'Print help (this message) and exit') { puts op; exit }
op.parse!

s3_enum = S3Enumerator.new(args)
s3_enum.run
exit if args[:preview_only]

app_name = args[:app_name]
eis_url = args[:eis_url]
s3_source_bucket_name = args[:s3_source_bucket_name]
s3_target_bucket_name = args[:s3_target_bucket_name]

eis_args = {
  url: eis_url,
  app_name: app_name,
  source_bucket_name: s3_source_bucket_name,
  target_bucket_name: s3_target_bucket_name,
  allow_reimport: args[:allow_reimport]
}

eis = Envoi::IngestService::Client.new(eis_args)

s3_enum.process_files do |object|
  full_path = File.join(s3_enum.s3_bucket_region, s3_source_bucket_name, object.key)
  raise "Error Creating Job for '#{full_path}':\n#{eis.response.body}" unless eis.job_create(object.key)
end



