#!/usr/bin/env ruby
lib_path = __FILE__ == '(irb)' ? File.join(Dir.cwd, 'lib') : File.expand_path('../../../lib', __FILE__)
$:.unshift(lib_path) unless $:.include?(lib_path) or !File.exists?(lib_path)
require 'rubygems'
require 'optparse'
require 'open3'

require 'envoi/mam/agent/cli'
require 'envoi/mam/vidispine/agent'

default_config_file_paths = Envoi::Mam::Agent::CLI::CONFIG_FILE_PATHS

aspera_ascp_paths = Envoi::Mam::Agent::TransferClient::Aspera::ASCP_PATHS
default_aspera_ascp_path = aspera_ascp_paths.find { |v| File.exist? v } || aspera_ascp_paths.first

current_command = ARGV.shift
ARGV << '--help' if ARGV.empty?

args = {
    :config_file_path  => default_config_file_paths,
    :default_ascp_path => default_aspera_ascp_path,
    :dry_run           => false,
    :operation         => :download,
    :preserve_path     => true,
    :transfer_type     => '',
}

args[:config_service_app_id] = ENV['ENVOI_MAM_AGENT_APP_ID']
args[:config_service_app_token] = ENV['ENVOI_MAM_AGENT_APP_TOKEN']
args[:config_service_app_url] = ENV['ENVOI_MAM_AGENT_APP_URL']

op = OptionParser.new
op.on('-c', '--config-file-path FILEPATH', 'The path to the configuration file.',
      "Default Path(s): #{args[:config_file_path]}") { |v| args[:config_file_path] = v }
op.on('--agent-app-id ID', '') { |v| args[:config_service_app_id] = v }
op.on('--agent-token TOKEN', '') { |v| args[:config_service_app_token] = v }
op.on('--agent-service-api URL', '') { |v| args[:config_service_api_url] = v }

op.on('-i', '--item-id ITEMID', 'The Vidispine item id.') { |v| args[:item_id] = v }
op.on('-s', '--shape-id SHAPEID', 'The Vidispine shape id.') { |v| args[:shape_id] = v }
op.on('-t', '--shape-tag SHAPETAG', 'The Vidispine shape tag used to find the shape when a shape id is not supplied.') { |v| args[:shape_tag] = v }
op.on('--storage-id STORAGEID', 'The Vidispine storage id to use for the transfer.') { |v| args[:storage_id] = v}

op.on('-d', '--destination-path DIRPATH', 'The destination path to download or upload to.',
      "default: '.'") { |v| args[:destination_path] = v }
op.on('-p','--[no-]preserve-path', "default: true") { |v| args[:preserve_path] = v }
op.on('-p', '--operation OP', [ :download, :upload ], 'Transfer Direction.',
      "default: #{args[:operation]}" ) { |v| args[:operation] = v }
op.on('--transfer-type TYPE', [ :aspera, :s3 ],'Force the type of transfer to use. Ex: aspera || s3') { |v| args[:transfer_type] = v }
op.on('--transfer-token TOKEN', '') { |v| args[:transfer_token] = v }
op.on('--dry-run', 'Run without executing the transfer.') { |v| args[:dry_run] = v }
op.on('--help', 'Show this message.') { puts op; exit }
op.load
op.parse!

args[:file_path] ||= ARGV.shift
args[:destination_path] ||= ARGV.shift unless ARGV.empty?

agent = Envoi::Mam::Vidispine::Agent.load_config_and_init(args)
agent.run_operation
