require 'cgi'
require 'open-uri'

require 'ubiquity/wiredrive/presentation_utility'
require 'ubiquity/envoi/api/utilities'

require 'envoi/mam/agent/cli'
require 'envoi/mam/wiredrive/agent'
require 'envoi/mam/iconik/agent'

current_command = ARGV.shift
ARGV << '--help' if ARGV.empty?

default_config_file_paths = Envoi::Mam::Agent::CLI::CONFIG_FILE_PATHS
# test_args = %w(--download --send-to-envoi --destination-path /tmp)
# ARGV.concat test_args
args = {
    :config_file_path => default_config_file_paths,
    :dry_run => false,
    :preserve_path => true,
    :destination_path => File.expand_path('.'),
    :should_download_presentation => false,
    :should_add_presentation_assets_to_envoi => false
}

args[:config_service_app_id] = ENV['ENVOI_MAM_AGENT_APP_ID']
args[:config_service_app_token] = ENV['ENVOI_MAM_AGENT_APP_TOKEN']
args[:config_service_app_url] = ENV['ENVOI_MAM_AGENT_APP_URL']

op = OptionParser.new
op.on('-c', '--config-file-path FILEPATH', 'The path to the configuration file.') { |v| args[:config_file_path] = v }
op.on('--agent-app-id ID', '') { |v| args[:config_service_app_id] = v }
op.on('--agent-token TOKEN', '') { |v| args[:config_service_app_token] = v }
op.on('--agent-service-api URL', '') { |v| args[:config_service_api_url] = v }

op.on('--presentation-invitation-token TOKEN', 'Presentation invitation token.') { |v| args[:presentation_invitation_token] = v }
op.on('--presentation-password PASSWORD', 'Presentation password') { |v| args[:presentation_password] = v }
op.on('--presentation-url URL', 'A shared presentation URL.') { |v| args[:presentation_url] = v }
# op.on('--presentation-id ID', '') { |v| args[:presentation_id] = v }
#
op.on('--[no-]download-presentation-assets', 'Determines if presentation asset files should be downloaded.') { |v| args[:should_download_presentation] = v }
op.on('--[no-]add-assets-to-envoi', 'Determines if presentation assets should be added to Envoi.') { |v| args[:should_add_presentation_assets_to_envoi] = v }
op.on('--[no-]add-assets-to-iconik', '') { |v| args[:should_add_presentation_assets_to_iconik] = v }
op.on('--iconik-storage-id STORAGEID', '') { |v| args[:iconik_storage_id] = v }

op.on('-d', '--destination-path DIRPATH', 'The destination path to download or upload to.', "default: '#{args[:destination_path]}'") { |v| args[:destination_path] = v }
op.on('-p','--[no-]preserve-path', "default: #{args[:preserve_path]}") { |v| args[:preserve_path] = v }

# op.on('--dry-run', 'Run without executing the transfer.') { |v| args[:dry_run] = v }
op.on('--help', 'Show this message.') { puts op; exit }
op.load
op.parse!

args[:destination_path] ||= ARGV.shift unless ARGV.empty?

presentation_url ||= args[:presentation_url] || ARGV.shift
if presentation_url
  uri = URI(presentation_url)
  presentation_invitation_token = uri.path
  presentation_url_token = presentation_invitation_token[1..-1] if presentation_invitation_token.start_with?('/')

  query_data = uri.query ? Hash[uri.query.split('&').map { |v| v.split('=') }] : { }
  query_password = query_data['password']
  presentation_url_password ||= query_password

  args[:presentation_invitation_token] ||= presentation_url_token
  args[:presentation_password] ||= presentation_url_password
end

class EnvoiImportUtility

  attr_accessor :initial_args, :api_client

  def initialize(args = { })
    @initial_args = args
    initialize_api_client(args)
  end

  def after_initialize
    @default_original_storage_id = args[:default_original_storage_id]
    @default_thumbnail_storage_id = args[:default_thumbnail_storage_id]
  end

  def initialize_api_client(args = { })
    @api_client = Ubiquity::Envoi::API::Utilities.new(args)
  end

  def import_wiredrive_presentation_asset(asset, options = { })
    title = asset['title'] || asset['label']
    description = asset['description']
    keywords = asset['keywords']

    media_elements = asset['media']

    media = media_elements.find { |me| me['type'] == 'original' }
    raise 'No original media found.' unless media
    url = media['url']
    uri = URI(url)
    mime_type = media['mimeType']
    size = media['fileSize']


    url_path = uri.path.split('?').first
    filename = File.basename(url_path)

    original_storage_id = options[:original_storage_id] || options[:storage_id] || @default_original_storage_id
    create_args = {
        :name => title,
        :description => description,
        :file => {
            :mime => 'video/mp4', # mime_type,
            :path => url,
            :name => filename,
            :storage_key => url_path,
            :setting_id => original_storage_id,
            'size' => size || 0
        }
    }
    r = api_client.media_file_create(create_args)

    media_file_id = r['id']
    raise 'No id found in response' unless media_file_id

    video_thumb = asset['thumbnails'].find { |e| e['category'] == 'max' }
    if video_thumb
      thumbnail_storage_id = options[:thumbnail_storage_id] || options[:storage_id] || @default_thumbnail_storage_id
      video_thumb_url = video_thumb['url']
      video_thumb_uri = URI(video_thumb_url)
      video_thumb_path = video_thumb_uri.path.split('?').first
      media_file_file_add_args = {
          'media_file_id' => media_file_id,
          'shape_type' => 'thumb',
          'shape_label' => 'Thumbnail',
          'name' => File.basename(video_thumb_path),
          'mime' => 'image/jpeg',
          'path' => video_thumb_url,
          'size' => 0,
          'setting_id' => thumbnail_storage_id,
          'storage_key' => video_thumb_path
      }
      r = api_client.media_file_file_add(media_file_file_add_args)

    end
  end

end

@args = args
def wdpu(args = @args); @wdpu ||= Ubiquity::Wiredrive::PresentationUtility.new(args) end

def presentation_assets_get(args = @args)
  presentation_invitation_token = args[:presentation_invitation_token]
  presentation_password = args[:presentation_password]
  wdpu.presentation_assets_get_using_token(presentation_invitation_token, presentation_password)
end
def presentation_assets; @presentation_assets ||= presentation_assets_get end

if args[:should_download_presentation]
  presentation_assets_download_results = wdpu.presentation_assets_download(assets: presentation_assets, destination_path: args[:destination_path], overwrite: args[:overwrite_on_download])
end

if args[:should_add_presentation_assets_to_envoi]

  wda = Envoi::Mam::Wiredrive::Agent.load_config_and_init(args)
  envoi_config = wda.config['envoi']
  envoi_http_host_address = envoi_config['http_host_address']
  envoi_username = envoi_config['username']
  envoi_password = envoi_config['password']
  envoi_original_storage_id = envoi_config['original_storage_id'] || envoi_config['default_storage_id']
  envoi_thumbnail_storage_id = envoi_config['thumbnail_storage_id'] || envoi_config['default_storage_id']
  envoi_args = { http_host_address: envoi_http_host_address, username: envoi_username, password: envoi_password }
  envoi = EnvoiImportUtility.new(envoi_args)
  presentation_assets.each do |a|
    begin
      envoi.import_wiredrive_presentation_asset(a, { original_storage_id: envoi_original_storage_id, thumbnail_storage_id: envoi_thumbnail_storage_id })
    rescue => e
      warn "#{e.message} #{$@.first}"
    end

  end
end

if args[:should_add_presentation_assets_to_iconik]
  presentation_assets_download_results ||= wdpu.presentation_assets_download(assets: presentation_assets, destination_path: '/tmp')

  # wda = Envoi::Mam::Wiredrive::Agent.load_config_and_init(args)
  # iconik_config = wda.config['iconik']

  iconik = Envoi::Mam::Iconik::Agent.load_config_and_init(args)
  iconik_storage_id = args[:iconik_storage_id]
  presentation_assets_download_results.each do |r|
    # asset = r[:asset]
    media_elements_downloaded = r[:media_element_download_responses]
    media_element_downloaded = media_elements_downloaded.first
    file_path = media_element_downloaded[:destination_file_path]
    begin
      # puts "WD FP: #{media_element_downloaded}"
      iconik.upload(file_path: file_path, storage_id: iconik_storage_id, destination_path: '/', preserve_path: false)
    rescue => e
      warn "#{e.message} #{$@.first}"
    end
  end

end

if args[:should_add_presentation_assets_to_vidispine]
  presentation_assets_download_results ||= wdpu.presentation_assets_download(assets: presentation_assets, destination_path: '/tmp')

  vidispine = Envoi::Mam::Vidispine::Agent.load_config_and_init(args)
  vidispine_storage_id = args[:vidispine_storage_id]
  presentation_assets_download_results.each do |r|
    # asset = r[:asset]
    media_elements_downloaded = r[:media_element_download_responses]
    media_element_downloaded = media_elements_downloaded.first
    file_path = media_element_downloaded[:destination_file_path]
    begin
      vidispine.upload(file_path: file_path, storage_id: vidispine_storage_id, destination_path: '/', preserve_path: false)
    rescue => e
      warn "#{e.message} #{$@.first}"
    end
  end

end